@@ -383,12 +383,6 @@ static void warn_multiple_config(const unsigned char *treeish_name,
 			commit_string, name, option);
 }
 
-static void warn_command_line_option(const char *var, const char *value)
-{
-	warning(_("ignoring '%s' which may be interpreted as"
-		  " a command-line option: %s"), var, value);
-}
-
 struct parse_config_parameter {
 	struct submodule_cache *cache;
 	const unsigned char *treeish_name;
@@ -396,13 +390,6 @@ struct parse_config_parameter {
 	int overwrite;
 };
 
-/*
- * Parse a config item from .gitmodules.
- *
- * This does not handle submodule-related configuration from the main
- * config store (.git/config, etc).  Callers are responsible for
- * checking for overrides in the main config store when appropriate.
- */
 static int parse_config(const char *var, const char *value, void *data)
 {
 	struct parse_config_parameter *me = data;
@@ -421,8 +408,6 @@ static int parse_config(const char *var, const char *value, void *data)
 	if (!strcmp(item.buf, "path")) {
 		if (!value)
 			ret = config_error_nonbool(var);
-		else if (looks_like_command_line_option(value))
-			warn_command_line_option(var, value);
 		else if (!me->overwrite && submodule->path)
 			warn_multiple_config(me->treeish_name, submodule->name,
 					"path");
@@ -463,8 +448,6 @@ static int parse_config(const char *var, const char *value, void *data)
 	} else if (!strcmp(item.buf, "url")) {
 		if (!value) {
 			ret = config_error_nonbool(var);
-		} else if (looks_like_command_line_option(value)) {
-			warn_command_line_option(var, value);
 		} else if (!me->overwrite && submodule->url) {
 			warn_multiple_config(me->treeish_name, submodule->name,
 					"url");
@@ -480,9 +463,8 @@ static int parse_config(const char *var, const char *value, void *data)
 			warn_multiple_config(me->treeish_name, submodule->name,
 					     "update");
 		else if (parse_submodule_update_strategy(value,
-			 &submodule->update_strategy) < 0 ||
-			 submodule->update_strategy.type == SM_UPDATE_COMMAND)
-			die(_("invalid value for %s"), var);
+			 &submodule->update_strategy) < 0)
+				die(_("invalid value for %s"), var);
 	} else if (!strcmp(item.buf, "shallow")) {
 		if (!me->overwrite && submodule->recommend_shallow != -1)
 			warn_multiple_config(me->treeish_name, submodule->name,
