@@ -26,7 +26,7 @@ test_expect_success 'start import with incomplete authors file' '
 test_expect_success 'imported 2 revisions successfully' '
 	(
 		cd x
-		test "`git rev-list refs/remotes/git-svn | wc -l`" -eq 2 &&
+		test "$(git rev-list refs/remotes/git-svn | wc -l)" -eq 2 &&
 		git rev-list -1 --pretty=raw refs/remotes/git-svn | \
 		  grep "^author BBBBBBB BBBBBBB <bb@example\.com> " &&
 		git rev-list -1 --pretty=raw refs/remotes/git-svn~1 | \
@@ -43,7 +43,7 @@ test_expect_success 'continues to import once authors have been added' '
 	(
 		cd x
 		git svn fetch --authors-file=../svn-authors &&
-		test "`git rev-list refs/remotes/git-svn | wc -l`" -eq 4 &&
+		test "$(git rev-list refs/remotes/git-svn | wc -l)" -eq 4 &&
 		git rev-list -1 --pretty=raw refs/remotes/git-svn | \
 		  grep "^author DDDDDDD DDDDDDD <dd@example\.com> " &&
 		git rev-list -1 --pretty=raw refs/remotes/git-svn~1 | \
@@ -73,8 +73,8 @@ tmp_config_get () {
 test_expect_success 'failure happened without negative side effects' '
 	(
 		cd aa-work &&
-		test 6 -eq "`tmp_config_get svn-remote.svn.branches-maxRev`" &&
-		test 6 -eq "`tmp_config_get svn-remote.svn.tags-maxRev`"
+		test 6 -eq "$(tmp_config_get svn-remote.svn.branches-maxRev)" &&
+		test 6 -eq "$(tmp_config_get svn-remote.svn.tags-maxRev)"
 	)
 	'
 
@@ -86,12 +86,12 @@ test_expect_success 'fetch continues after authors-file is fixed' '
 	(
 		cd aa-work &&
 		git svn fetch --authors-file=../svn-authors &&
-		test 8 -eq "`tmp_config_get svn-remote.svn.branches-maxRev`" &&
-		test 8 -eq "`tmp_config_get svn-remote.svn.tags-maxRev`"
+		test 8 -eq "$(tmp_config_get svn-remote.svn.branches-maxRev)" &&
+		test 8 -eq "$(tmp_config_get svn-remote.svn.tags-maxRev)"
 	)
 	'
 
-test_expect_success 'fresh clone with svn.authors-file in config' '
+test_expect_success !MINGW 'fresh clone with svn.authors-file in config' '
 	(
 		rm -r "$GIT_DIR" &&
 		test x = x"$(git config svn.authorsfile)" &&
